@@ -16,6 +16,9 @@
 #  under the License.
 
 from types import ModuleType
+
+from pyspark.sql.types import FloatType, DoubleType, IntegerType, LongType, DecimalType, ShortType, ByteType
+
 from sedona.maps.SedonaMapUtils import SedonaMapUtils
 
 
@@ -24,7 +27,7 @@ class SedonaPyDeck:
     # User Facing APIs
     @classmethod
     def create_choropleth_map(cls, df, fill_color=None, plot_col=None, initial_view_state=None, map_style=None,
-                              map_provider=None, elevation_col=0):
+                              map_provider=None, elevation_col=0, api_keys=None, stroked=True):
         """
         Create a pydeck map with a choropleth layer added
         :param elevation_col: Optional elevation for the polygons
@@ -35,8 +38,17 @@ class SedonaPyDeck:
         :param initial_view_state:
         :param map_style:
         :param map_provider:
+        :param api_keys: Optional dictionary of API keys for Map providers
         :return: A pydeck Map object with choropleth layer added:
         """
+
+        for field in df.schema.fields:
+            if field.name == plot_col and field.dataType not in [IntegerType(), FloatType(), DoubleType(), LongType(),
+                                                                 DecimalType(), ShortType(), ByteType()]:
+                message = (f"'{field.name}' must be of numeric type. \nIf you are importing data from csv set "
+                           f"'inferSchema' option as true")
+                raise TypeError(message) from None
+
         pdk = _try_import_pydeck()
 
         if initial_view_state is None:
@@ -55,20 +67,19 @@ class SedonaPyDeck:
             get_fill_color=fill_color,
             opacity=1.0,
             get_elevation=elevation_col,
-            stroked=False,
+            stroked=stroked,
             extruded=True,
             wireframe=True,
             pickable=True
         )
 
-        map_ = pdk.Deck(layers=[choropleth_layer], initial_view_state=initial_view_state)
-        SedonaPyDeck._set_optional_parameters_(p_map=map_, map_style=map_style, map_provider=map_provider)
-        return map_
+        return SedonaPyDeck._create_map_obj(layer=choropleth_layer, initial_view_state=initial_view_state, map_style=map_style,
+                                            map_provider=map_provider, api_keys=api_keys)
 
     @classmethod
     def create_geometry_map(cls, df, fill_color="[85, 183, 177, 255]", line_color="[85, 183, 177, 255]",
                             elevation_col=0, initial_view_state=None,
-                            map_style=None, map_provider=None):
+                            map_style=None, map_provider=None, api_keys=None, stroked=True):
         """
         Create a pydeck map with a GeoJsonLayer added for plotting given geometries.
         :param line_color:
@@ -78,6 +89,7 @@ class SedonaPyDeck:
         :param initial_view_state: optional initial view state of the pydeck map
         :param map_style: optional map_style of the pydeck map
         :param map_provider: optional map_provider of the pydeck map
+        :param api_keys: Optional dictionary of API keys for Map providers
         :return: A pydeck map with a GeoJsonLayer map added
         """
         pdk = _try_import_pydeck()
@@ -91,21 +103,21 @@ class SedonaPyDeck:
         #         line_color = "[237, 119, 79]"
 
         layer = SedonaPyDeck._create_fat_layer_(gdf, fill_color=fill_color, elevation_col=elevation_col,
-                                                line_color=line_color)
+                                                line_color=line_color, stroked=stroked)
 
         if initial_view_state is None:
             initial_view_state = pdk.data_utils.compute_view(gdf['coordinate_array_sedona'])
 
         if elevation_col != 0 and geom_type == "Polygon":
             initial_view_state.pitch = 45  # If polygons are elevated, change the pitch to visualize the elevation better
-        map_ = pdk.Deck(layers=[layer], map_style='dark', initial_view_state=initial_view_state)
-        SedonaPyDeck._set_optional_parameters_(p_map=map_, map_style=map_style, map_provider=map_provider)
-        return map_
+
+        return SedonaPyDeck._create_map_obj(layer=layer, initial_view_state=initial_view_state, map_style=map_style,
+                                            map_provider=map_provider, api_keys=api_keys)
 
     @classmethod
     def create_scatterplot_map(cls, df, fill_color="[255, 140, 0]", radius_col=1, radius_min_pixels=1,
                                radius_max_pixels=10, radius_scale=1, initial_view_state=None, map_style=None,
-                               map_provider=None):
+                               map_provider=None, api_keys=None):
         """
         Create a pydeck map with a scatterplot layer
         :param radius_scale:
@@ -117,6 +129,7 @@ class SedonaPyDeck:
         :param initial_view_state: optional initial view state of a pydeck map
         :param map_style: optional map_style to be added to the pydeck map
         :param map_provider: optional map_provider to be added to the pydeck map
+        :param api_keys: Optional dictionary of API keys for Map providers
         :return: A pydeck map object with a scatterplot layer added
         """
         pdk = _try_import_pydeck()
@@ -139,13 +152,12 @@ class SedonaPyDeck:
         if initial_view_state is None:
             initial_view_state = pdk.data_utils.compute_view(gdf['coordinate_array_sedona'])
 
-        map_ = pdk.Deck(layers=[layer], initial_view_state=initial_view_state)
-        SedonaPyDeck._set_optional_parameters_(p_map=map_, map_style=map_style, map_provider=map_provider)
-        return map_
+        return SedonaPyDeck._create_map_obj(layer=layer, initial_view_state=initial_view_state, map_style=map_style,
+                                            map_provider=map_provider, api_keys=api_keys)
 
     @classmethod
     def create_heatmap(cls, df, color_range=None, weight=1, aggregation="SUM", initial_view_state=None, map_style=None,
-                       map_provider=None):
+                       map_provider=None, api_keys=None):
         """
         Create a pydeck map with a heatmap layer added
         :param df: SedonaDataFrame to be used to plot the heatmap
@@ -155,6 +167,7 @@ class SedonaPyDeck:
         :param initial_view_state: Optional initial view state of the pydeck map
         :param map_style: Optional map_style of the pydeck map
         :param map_provider: Optional map_provider for the pydeck map
+        :param api_keys: Optional dictionary of API keys for Map providers
         :return: A pydeck map with a heatmap layer added
         """
         pdk = _try_import_pydeck()
@@ -186,10 +199,8 @@ class SedonaPyDeck:
         if initial_view_state is None:
             initial_view_state = pdk.data_utils.compute_view(gdf['coordinate_array_sedona'])
 
-        map_ = pdk.Deck(layers=[layer], initial_view_state=initial_view_state)
-        SedonaPyDeck._set_optional_parameters_(p_map=map_, map_style=map_style, map_provider=map_provider)
-
-        return map_
+        return SedonaPyDeck._create_map_obj(layer=layer, initial_view_state=initial_view_state, map_style=map_style,
+                                            map_provider=map_provider, api_keys=api_keys)
 
     @classmethod
     def _prepare_df_(cls, df, add_coords=False, geometry_col=None):
@@ -206,21 +217,6 @@ class SedonaPyDeck:
             SedonaPyDeck._create_coord_column_(gdf=gdf, geometry_col=geometry_col)
         return gdf
 
-    # Utility APIs specific to SedonaPyDeck
-    @classmethod
-    def _set_optional_parameters_(cls, p_map, map_style, map_provider):
-        """
-        Set optional parameters to a given pydeck map
-        (Parameters are self-explanatory for a pydeck map)
-        :param p_map:
-        :param map_style:
-        :param map_provider:
-        """
-        if map_style is not None:
-            p_map.map_style = map_style
-
-        if map_provider is not None:
-            p_map.map_provider = map_provider
 
     @classmethod
     def _create_default_fill_color_(cls, gdf, plot_col):
@@ -244,7 +240,7 @@ class SedonaPyDeck:
             lambda val: list(SedonaMapUtils.__extract_coordinate__(val[geometry_col], type_list)), axis=1)
 
     @classmethod
-    def _create_fat_layer_(cls, gdf, fill_color, line_color, elevation_col):
+    def _create_fat_layer_(cls, gdf, fill_color, line_color, elevation_col, stroked):
         pdk = _try_import_pydeck()
         layer = pdk.Layer(
             'GeoJsonLayer',  # `type` positional argument is here
@@ -252,7 +248,7 @@ class SedonaPyDeck:
             auto_highlight=True,
             get_fill_color=fill_color,
             opacity=0.4,
-            stroked=False,
+            stroked=stroked,
             extruded=True,
             get_elevation=elevation_col,
             get_line_color=line_color,
@@ -262,6 +258,27 @@ class SedonaPyDeck:
 
         return layer
 
+    # creates the final map object and handles the parameters for pydeck
+    @classmethod
+    def _create_map_obj(cls, layer, initial_view_state, map_style, map_provider, api_keys):
+        pdk = _try_import_pydeck()
+
+        if map_provider is None:
+            map_provider = 'carto'
+
+        if map_style is None:
+            map_style = 'dark'
+
+        # Default to mapbox if user selects 'satellite' map_style as pydeck uses mapbox and google maps for satellite basemap
+        if map_style == 'satellite' and map_provider != 'google_maps':
+            map_provider = 'mapbox'
+
+        if api_keys is not None:
+            api_keys = api_keys
+
+        return pdk.Deck(layers=[layer], initial_view_state=initial_view_state, map_style=map_style,
+                        map_provider=map_provider, api_keys=api_keys)
+
 
 def _try_import_pydeck() -> ModuleType:
     try:
