@@ -16,7 +16,6 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-
 package org.apache.spark.sql.sedona_sql.expressions.raster
 
 import org.apache.spark.sql.catalyst.InternalRow
@@ -34,23 +33,26 @@ import java.util.Base64
 import javax.imageio.ImageIO
 
 case class RS_Array(inputExpressions: Seq[Expression])
-  extends Expression with ImplicitCastInputTypes with CodegenFallback with UserDataGeneratator {
+    extends Expression
+    with ImplicitCastInputTypes
+    with CodegenFallback
+    with UserDataGeneratator {
   override def nullable: Boolean = false
 
   override def eval(inputRow: InternalRow): Any = {
     // This is an expression which takes one input expressions
     assert(inputExpressions.length == 2)
-    val len =inputExpressions(0).eval(inputRow).asInstanceOf[Int]
+    val len = inputExpressions(0).eval(inputRow).asInstanceOf[Int]
     val num = inputExpressions(1).eval(inputRow).asInstanceOf[Double]
     val result = createarray(len, num)
     new GenericArrayData(result)
   }
 
   // Generate an empty band for the given spectral band in ageotiff image
-  private def createarray(len:Int, num:Double):Array[Double] = {
+  private def createarray(len: Int, num: Double): Array[Double] = {
 
     val result = new Array[Double](len)
-    for(i<-0 until len) {
+    for (i <- 0 until len) {
       result(i) = num
     }
     result
