@@ -16,7 +16,6 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-
 package org.apache.spark.sql.sedona_sql.expressions.raster
 
 import org.apache.sedona.common.raster.RasterBandAccessors
@@ -30,57 +29,48 @@ import org.apache.spark.sql.sedona_sql.expressions.raster.implicits.RasterInputE
 import org.apache.spark.sql.sedona_sql.expressions.InferredExpression
 import org.geotools.coverage.grid.GridCoverage2D
 
-case class RS_BandNoDataValue(inputExpressions: Seq[Expression]) extends InferredExpression(inferrableFunction2(RasterBandAccessors.getBandNoDataValue), inferrableFunction1(RasterBandAccessors.getBandNoDataValue)) {
+case class RS_BandNoDataValue(inputExpressions: Seq[Expression])
+    extends InferredExpression(
+      inferrableFunction2(RasterBandAccessors.getBandNoDataValue),
+      inferrableFunction1(RasterBandAccessors.getBandNoDataValue)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
-case class RS_Count(inputExpressions: Seq[Expression]) extends InferredExpression(
-  inferrableFunction2(RasterBandAccessors.getCount), inferrableFunction1(RasterBandAccessors.getCount),
-    inferrableFunction3(RasterBandAccessors.getCount)) {
-    protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
-      copy(inputExpressions = newChildren)
-    }
-  }
-
-case class RS_ZonalStats(inputExpressions: Seq[Expression]) extends InferredExpression(
-  inferrableFunction6(RasterBandAccessors.getZonalStats),
-  inferrableFunction5(RasterBandAccessors.getZonalStats),
-  inferrableFunction4(RasterBandAccessors.getZonalStats),
-  inferrableFunction3(RasterBandAccessors.getZonalStats)
-) {
+case class RS_Count(inputExpressions: Seq[Expression])
+    extends InferredExpression(
+      inferrableFunction2(RasterBandAccessors.getCount),
+      inferrableFunction1(RasterBandAccessors.getCount),
+      inferrableFunction3(RasterBandAccessors.getCount)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
-case class RS_ZonalStatsAll(inputExpressions: Seq[Expression]) extends InferredExpression(
-  inferrableFunction2(RasterBandAccessors.getZonalStatsAll), inferrableFunction4(RasterBandAccessors.getZonalStatsAll),
-  inferrableFunction3(RasterBandAccessors.getZonalStatsAll), inferrableFunction5(RasterBandAccessors.getZonalStatsAll)
-) {
+case class RS_ZonalStats(inputExpressions: Seq[Expression])
+    extends InferredExpression(
+      inferrableFunction6(RasterBandAccessors.getZonalStats),
+      inferrableFunction5(RasterBandAccessors.getZonalStats),
+      inferrableFunction4(RasterBandAccessors.getZonalStats),
+      inferrableFunction3(RasterBandAccessors.getZonalStats)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
-case class RS_SummaryStats(inputExpressions: Seq[Expression]) extends InferredExpression(
-  inferrableFunction2(RasterBandAccessors.getSummaryStats), inferrableFunction3(RasterBandAccessors.getSummaryStats),
-  inferrableFunction4(RasterBandAccessors.getSummaryStats)) {
+case class RS_SummaryStats(inputExpressions: Seq[Expression])
+    extends InferredExpression(
+      inferrableFunction2(RasterBandAccessors.getSummaryStats),
+      inferrableFunction3(RasterBandAccessors.getSummaryStats),
+      inferrableFunction4(RasterBandAccessors.getSummaryStats)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
-case class RS_SummaryStatsAll(inputExpressions: Seq[Expression]) extends InferredExpression(
-  inferrableFunction1(RasterBandAccessors.getSummaryStatsAll), inferrableFunction2(RasterBandAccessors.getSummaryStatsAll),
-  inferrableFunction3(RasterBandAccessors.getSummaryStatsAll)) {
-  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
-    copy(inputExpressions = newChildren)
-  }
-}
-
-case class RS_Band(inputExpressions: Seq[Expression]) extends InferredExpression(RasterBandAccessors.getBand _) {
+case class RS_Band(inputExpressions: Seq[Expression])
+    extends InferredExpression(RasterBandAccessors.getBand _) {
 
   override def evalWithoutSerialization(input: InternalRow): Any = {
     val raster = inputExpressions.head.toRaster(input)
@@ -102,15 +92,19 @@ case class RS_Band(inputExpressions: Seq[Expression]) extends InferredExpression
   }
 }
 
-case class RS_BandPixelType(inputExpressions: Seq[Expression]) extends InferredExpression(inferrableFunction2(RasterBandAccessors.getBandType), inferrableFunction1(RasterBandAccessors.getBandType)) {
+case class RS_BandPixelType(inputExpressions: Seq[Expression])
+    extends InferredExpression(
+      inferrableFunction2(RasterBandAccessors.getBandType),
+      inferrableFunction1(RasterBandAccessors.getBandType)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
 case class RS_BandIsNoData(inputExpressions: Seq[Expression])
-  extends InferredExpression(inferrableFunction2(RasterBandAccessors.bandIsNoData),
-    inferrableFunction1(RasterBandAccessors.bandIsNoData)) {
+    extends InferredExpression(
+      inferrableFunction2(RasterBandAccessors.bandIsNoData),
+      inferrableFunction1(RasterBandAccessors.bandIsNoData)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
