@@ -32,7 +32,7 @@ import org.apache.spark.sql.sedona_sql.expressions.raster.implicits.{RasterEnhan
 import org.apache.spark.sql.types.{ArrayType, BooleanType, Decimal, IntegerType, NullType, StructType}
 
 case class RS_FromArcInfoAsciiGrid(inputExpressions: Seq[Expression])
-  extends InferredExpression(RasterConstructors.fromArcInfoAsciiGrid _) {
+    extends InferredExpression(RasterConstructors.fromArcInfoAsciiGrid _) {
   override def foldable: Boolean = false
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
@@ -40,17 +40,19 @@ case class RS_FromArcInfoAsciiGrid(inputExpressions: Seq[Expression])
   }
 }
 
-case class RS_AsRaster(inputExpressions: Seq[Expression]) extends InferredExpression(
-  inferrableFunction5(RasterConstructors.asRaster), inferrableFunction3(RasterConstructors.asRaster),
-  inferrableFunction4(RasterConstructors.asRaster), inferrableFunction6(RasterConstructors.asRaster)
-) {
+case class RS_AsRaster(inputExpressions: Seq[Expression])
+    extends InferredExpression(
+      inferrableFunction5(RasterConstructors.asRaster),
+      inferrableFunction3(RasterConstructors.asRaster),
+      inferrableFunction4(RasterConstructors.asRaster),
+      inferrableFunction6(RasterConstructors.asRaster)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
 case class RS_FromGeoTiff(inputExpressions: Seq[Expression])
-  extends InferredExpression(RasterConstructors.fromGeoTiff _) {
+    extends InferredExpression(RasterConstructors.fromGeoTiff _) {
 
   override def foldable: Boolean = false
 
@@ -60,9 +62,11 @@ case class RS_FromGeoTiff(inputExpressions: Seq[Expression])
 }
 
 case class RS_MakeEmptyRaster(inputExpressions: Seq[Expression])
-  extends InferredExpression(
-    inferrableFunction6(RasterConstructors.makeEmptyRaster), inferrableFunction7(RasterConstructors.makeEmptyRaster),
-    inferrableFunction10(RasterConstructors.makeEmptyRaster), inferrableFunction11(RasterConstructors.makeEmptyRaster)) {
+    extends InferredExpression(
+      inferrableFunction6(RasterConstructors.makeEmptyRaster),
+      inferrableFunction7(RasterConstructors.makeEmptyRaster),
+      inferrableFunction10(RasterConstructors.makeEmptyRaster),
+      inferrableFunction11(RasterConstructors.makeEmptyRaster)) {
 
   override def foldable: Boolean = false
 
@@ -72,25 +76,25 @@ case class RS_MakeEmptyRaster(inputExpressions: Seq[Expression])
 }
 
 case class RS_MakeRaster(inputExpressions: Seq[Expression])
-  extends InferredExpression(inferrableFunction3(RasterConstructors.makeNonEmptyRaster)) {
+    extends InferredExpression(inferrableFunction3(RasterConstructors.makeNonEmptyRaster)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
 case class RS_MakeRasterForTesting(inputExpressions: Seq[Expression])
-  extends InferredExpression(RasterConstructorsForTesting.makeRasterForTesting _) {
+    extends InferredExpression(RasterConstructorsForTesting.makeRasterForTesting _) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
 case class RS_Tile(inputExpressions: Seq[Expression])
-  extends InferredExpression(
-    nullTolerantInferrableFunction3(RasterConstructors.rsTile),
-    nullTolerantInferrableFunction4(RasterConstructors.rsTile),
-    nullTolerantInferrableFunction5(RasterConstructors.rsTile),
-    nullTolerantInferrableFunction6(RasterConstructors.rsTile)) {
+    extends InferredExpression(
+      nullTolerantInferrableFunction3(RasterConstructors.rsTile),
+      nullTolerantInferrableFunction4(RasterConstructors.rsTile),
+      nullTolerantInferrableFunction5(RasterConstructors.rsTile),
+      nullTolerantInferrableFunction6(RasterConstructors.rsTile)) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
@@ -115,9 +119,17 @@ case class RS_TileExplode(children: Seq[Expression]) extends Generator with Code
         case value: Decimal => value.toDouble
         case value: Float => value.toDouble
         case value: Double => value
-        case value: Any => throw new IllegalArgumentException("Unsupported class for noDataValue: " + value.getClass)
+        case value: Any =>
+          throw new IllegalArgumentException(
+            "Unsupported class for noDataValue: " + value.getClass)
       }
-      val tiles = RasterConstructors.generateTiles(raster, bandIndices, tileWidth, tileHeight, padWithNoDataValue, noDataValue)
+      val tiles = RasterConstructors.generateTiles(
+        raster,
+        bandIndices,
+        tileWidth,
+        tileHeight,
+        padWithNoDataValue,
+        noDataValue)
       tiles.map { tile =>
         val gridCoverage2D = tile.getCoverage
         val row = InternalRow(tile.getTileX, tile.getTileY, gridCoverage2D.serialize)
@@ -142,12 +154,13 @@ case class RS_TileExplode(children: Seq[Expression]) extends Generator with Code
 }
 
 object RS_TileExplode {
-  case class Arguments(rasterExpr: Expression,
-                       bandIndicesExpr: Expression,
-                       tileWidthExpr: Expression,
-                       tileHeightExpr: Expression,
-                       padWithNoDataExpr: Expression,
-                       noDataValExpr: Expression)
+  case class Arguments(
+      rasterExpr: Expression,
+      bandIndicesExpr: Expression,
+      tileWidthExpr: Expression,
+      tileHeightExpr: Expression,
+      padWithNoDataExpr: Expression,
+      noDataValExpr: Expression)
 
   def arguments(inputExpressions: Seq[Expression]): Arguments = {
     // RS_Tile/RS_TileExplode has 3 forms:
@@ -192,8 +205,9 @@ object RS_TileExplode {
 }
 
 case class RS_FromNetCDF(inputExpressions: Seq[Expression])
-  extends InferredExpression(
-    inferrableFunction2(RasterConstructors.fromNetCDF), inferrableFunction4(RasterConstructors.fromNetCDF)) {
+    extends InferredExpression(
+      inferrableFunction2(RasterConstructors.fromNetCDF),
+      inferrableFunction4(RasterConstructors.fromNetCDF)) {
 
   override def foldable: Boolean = false
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
@@ -202,7 +216,7 @@ case class RS_FromNetCDF(inputExpressions: Seq[Expression])
 }
 
 case class RS_NetCDFInfo(inputExpressions: Seq[Expression])
-  extends InferredExpression(RasterConstructors.getRecordInfo _) {
+    extends InferredExpression(RasterConstructors.getRecordInfo _) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
