@@ -27,8 +27,7 @@ import org.apache.spark.SparkException
 import org.apache.spark.scheduler.{SparkListener, SparkListenerTaskEnd}
 import org.apache.spark.sql.Row
 import org.apache.spark.sql.SaveMode
-import org.apache.spark.sql.execution.datasources.parquet.GeoParquetMetaData
-import org.apache.spark.sql.execution.datasources.parquet.ParquetReadSupport
+import org.apache.spark.sql.execution.datasources.parquet.{Covering, GeoParquetMetaData, ParquetReadSupport}
 import org.apache.spark.sql.functions.{col, expr}
 import org.apache.spark.sql.sedona_sql.UDT.GeometryUDT
 import org.apache.spark.sql.sedona_sql.expressions.st_constructors.{ST_Point, ST_PolygonFromEnvelope}
@@ -51,13 +50,15 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
   val geoparquetdatalocation2: String = resourceFolder + "geoparquet/example2.parquet"
   val geoparquetdatalocation3: String = resourceFolder + "geoparquet/example3.parquet"
   val geoparquetdatalocation4: String = resourceFolder + "geoparquet/example-1.0.0-beta.1.parquet"
-  val legacyparquetdatalocation: String = resourceFolder + "parquet/legacy-parquet-nested-columns.snappy.parquet"
+  val geoparquetdatalocation5: String = resourceFolder + "geoparquet/example-1.1.0.parquet"
+  val legacyparquetdatalocation: String =
+    resourceFolder + "parquet/legacy-parquet-nested-columns.snappy.parquet"
   val geoparquetoutputlocation: String = resourceFolder + "geoparquet/geoparquet_output/"
 
   override def afterAll(): Unit = FileUtils.deleteDirectory(new File(geoparquetoutputlocation))
 
-  describe("GeoParquet IO tests"){
-    it("GEOPARQUET Test example1 i.e. naturalearth_lowers dataset's Read and Write"){
+  describe("GeoParquet IO tests") {
+    it("GEOPARQUET Test example1 i.e. naturalearth_lowers dataset's Read and Write") {
       val df = sparkSession.read.format("geoparquet").load(geoparquetdatalocation1)
       val rows = df.collect()(0)
       assert(rows.getAs[Long]("pop_est") == 920938)
@@ -65,35 +66,63 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
       assert(rows.getAs[String]("name") == "Fiji")
       assert(rows.getAs[String]("iso_a3") == "FJI")
       assert(rows.getAs[Double]("gdp_md_est") == 8374.0)
-      assert(rows.getAs[Geometry]("geometry").toString == "MULTIPOLYGON (((180 -16.067132663642447, 180 -16.555216566639196, 179.36414266196414 -16.801354076946883, 178.72505936299711 -17.01204167436804, 178.59683859511713 -16.639150000000004, 179.0966093629971 -16.433984277547403, 179.4135093629971 -16.379054277547404, 180 -16.067132663642447)), ((178.12557 -17.50481, 178.3736 -17.33992, 178.71806 -17.62846, 178.55271 -18.15059, 177.93266000000003 -18.28799, 177.38146 -18.16432, 177.28504 -17.72465, 177.67087 -17.381140000000002, 178.12557 -17.50481)), ((-179.79332010904864 -16.020882256741224, -179.9173693847653 -16.501783135649397, -180 -16.555216566639196, -180 -16.067132663642447, -179.79332010904864 -16.020882256741224)))")
-      df.write.format("geoparquet").mode(SaveMode.Overwrite).save(geoparquetoutputlocation + "/gp_sample1.parquet")
-      val df2 = sparkSession.read.format("geoparquet").load(geoparquetoutputlocation + "/gp_sample1.parquet")
+      assert(
+        rows
+          .getAs[Geometry]("geometry")
+          .toString == "MULTIPOLYGON (((180 -16.067132663642447, 180 -16.555216566639196, 179.36414266196414 -16.801354076946883, 178.72505936299711 -17.01204167436804, 178.59683859511713 -16.639150000000004, 179.0966093629971 -16.433984277547403, 179.4135093629971 -16.379054277547404, 180 -16.067132663642447)), ((178.12557 -17.50481, 178.3736 -17.33992, 178.71806 -17.62846, 178.55271 -18.15059, 177.93266000000003 -18.28799, 177.38146 -18.16432, 177.28504 -17.72465, 177.67087 -17.381140000000002, 178.12557 -17.50481)), ((-179.79332010904864 -16.020882256741224, -179.9173693847653 -16.501783135649397, -180 -16.555216566639196, -180 -16.067132663642447, -179.79332010904864 -16.020882256741224)))")
+      df.write
+        .format("geoparquet")
+        .mode(SaveMode.Overwrite)
+        .save(geoparquetoutputlocation + "/gp_sample1.parquet")
+      val df2 = sparkSession.read
+        .format("geoparquet")
+        .load(geoparquetoutputlocation + "/gp_sample1.parquet")
       val newrows = df2.collect()(0)
-      assert(newrows.getAs[Geometry]("geometry").toString == "MULTIPOLYGON (((180 -16.067132663642447, 180 -16.555216566639196, 179.36414266196414 -16.801354076946883, 178.72505936299711 -17.01204167436804, 178.59683859511713 -16.639150000000004, 179.0966093629971 -16.433984277547403, 179.4135093629971 -16.379054277547404, 180 -16.067132663642447)), ((178.12557 -17.50481, 178.3736 -17.33992, 178.71806 -17.62846, 178.55271 -18.15059, 177.93266000000003 -18.28799, 177.38146 -18.16432, 177.28504 -17.72465, 177.67087 -17.381140000000002, 178.12557 -17.50481)), ((-179.79332010904864 -16.020882256741224, -179.9173693847653 -16.501783135649397, -180 -16.555216566639196, -180 -16.067132663642447, -179.79332010904864 -16.020882256741224)))")
+      assert(
+        newrows
+          .getAs[Geometry]("geometry")
+          .toString == "MULTIPOLYGON (((180 -16.067132663642447, 180 -16.555216566639196, 179.36414266196414 -16.801354076946883, 178.72505936299711 -17.01204167436804, 178.59683859511713 -16.639150000000004, 179.0966093629971 -16.433984277547403, 179.4135093629971 -16.379054277547404, 180 -16.067132663642447)), ((178.12557 -17.50481, 178.3736 -17.33992, 178.71806 -17.62846, 178.55271 -18.15059, 177.93266000000003 -18.28799, 177.38146 -18.16432, 177.28504 -17.72465, 177.67087 -17.381140000000002, 178.12557 -17.50481)), ((-179.79332010904864 -16.020882256741224, -179.9173693847653 -16.501783135649397, -180 -16.555216566639196, -180 -16.067132663642447, -179.79332010904864 -16.020882256741224)))")
     }
-    it("GEOPARQUET Test example2 i.e. naturalearth_citie dataset's Read and Write"){
+    it("GEOPARQUET Test example2 i.e. naturalearth_citie dataset's Read and Write") {
       val df = sparkSession.read.format("geoparquet").load(geoparquetdatalocation2)
       val rows = df.collect()(0)
       assert(rows.getAs[String]("name") == "Vatican City")
-      assert(rows.getAs[Geometry]("geometry").toString == "POINT (12.453386544971766 41.903282179960115)")
-      df.write.format("geoparquet").mode(SaveMode.Overwrite).save(geoparquetoutputlocation + "/gp_sample2.parquet")
-      val df2 = sparkSession.read.format("geoparquet").load(geoparquetoutputlocation + "/gp_sample2.parquet")
+      assert(
+        rows
+          .getAs[Geometry]("geometry")
+          .toString == "POINT (12.453386544971766 41.903282179960115)")
+      df.write
+        .format("geoparquet")
+        .mode(SaveMode.Overwrite)
+        .save(geoparquetoutputlocation + "/gp_sample2.parquet")
+      val df2 = sparkSession.read
+        .format("geoparquet")
+        .load(geoparquetoutputlocation + "/gp_sample2.parquet")
       val newrows = df2.collect()(0)
       assert(newrows.getAs[String]("name") == "Vatican City")
-      assert(newrows.getAs[Geometry]("geometry").toString == "POINT (12.453386544971766 41.903282179960115)")
+      assert(
+        newrows
+          .getAs[Geometry]("geometry")
+          .toString == "POINT (12.453386544971766 41.903282179960115)")
     }
-    it("GEOPARQUET Test example3 i.e. nybb dataset's Read and Write"){
+    it("GEOPARQUET Test example3 i.e. nybb dataset's Read and Write") {
       val df = sparkSession.read.format("geoparquet").load(geoparquetdatalocation3)
       val rows = df.collect()(0)
       assert(rows.getAs[Long]("BoroCode") == 5)
       assert(rows.getAs[String]("BoroName") == "Staten Island")
       assert(rows.getAs[Double]("Shape_Leng") == 330470.010332)
-      assert(rows.getAs[Double]("Shape_Area") == 1.62381982381E9)
+      assert(rows.getAs[Double]("Shape_Area") == 1.62381982381e9)
       assert(rows.getAs[Geometry]("geometry").toString.startsWith("MULTIPOLYGON (((970217.022"))
-      df.write.format("geoparquet").mode(SaveMode.Overwrite).save(geoparquetoutputlocation + "/gp_sample3.parquet")
-      val df2 = sparkSession.read.format("geoparquet").load(geoparquetoutputlocation + "/gp_sample3.parquet")
+      df.write
+        .format("geoparquet")
+        .mode(SaveMode.Overwrite)
+        .save(geoparquetoutputlocation + "/gp_sample3.parquet")
+      val df2 = sparkSession.read
+        .format("geoparquet")
+        .load(geoparquetoutputlocation + "/gp_sample3.parquet")
       val newrows = df2.collect()(0)
-      assert(newrows.getAs[Geometry]("geometry").toString.startsWith("MULTIPOLYGON (((970217.022"))
+      assert(
+        newrows.getAs[Geometry]("geometry").toString.startsWith("MULTIPOLYGON (((970217.022"))
     }
     it("GEOPARQUET Test example-1.0.0-beta.1.parquet") {
       val df = sparkSession.read.format("geoparquet").load(geoparquetdatalocation4)
@@ -110,11 +139,14 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
       assert(newRows(0).getAs[AnyRef]("geometry").isInstanceOf[Geometry])
       assert(rows sameElements newRows)
 
-      val parquetFiles = new File(geoParquetSavePath).listFiles().filter(_.getName.endsWith(".parquet"))
+      val parquetFiles =
+        new File(geoParquetSavePath).listFiles().filter(_.getName.endsWith(".parquet"))
       parquetFiles.foreach { filePath =>
-        val metadata = ParquetFileReader.open(
-          HadoopInputFile.fromPath(new Path(filePath.getPath), new Configuration()))
-          .getFooter.getFileMetaData.getKeyValueMetaData
+        val metadata = ParquetFileReader
+          .open(HadoopInputFile.fromPath(new Path(filePath.getPath), new Configuration()))
+          .getFooter
+          .getFileMetaData
+          .getKeyValueMetaData
         assert(metadata.containsKey("geo"))
         val geo = parseJson(metadata.get("geo"))
         implicit val formats: org.json4s.Formats = org.json4s.DefaultFormats
@@ -126,26 +158,49 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
         assert(!sparkSqlRowMetadata.contains("GeometryUDT"))
       }
     }
+    it("GEOPARQUET Test example-1.1.0.parquet") {
+      val df = sparkSession.read.format("geoparquet").load(geoparquetdatalocation5)
+      val count = df.count()
+      val rows = df.collect()
+      assert(rows(0).getAs[AnyRef]("geometry").isInstanceOf[Geometry])
+      assert(count == rows.length)
+
+      val geoParquetSavePath = geoparquetoutputlocation + "/gp_sample5.parquet"
+      df.write.format("geoparquet").mode(SaveMode.Overwrite).save(geoParquetSavePath)
+      val df2 = sparkSession.read.format("geoparquet").load(geoParquetSavePath)
+      val newRows = df2.collect()
+      assert(rows.length == newRows.length)
+      assert(newRows(0).getAs[AnyRef]("geometry").isInstanceOf[Geometry])
+      assert(rows sameElements newRows)
+    }
 
     it("GeoParquet with multiple geometry columns") {
       val wktReader = new WKTReader()
       val testData = Seq(
-        Row(1, wktReader.read("POINT (1 2)"), wktReader.read("POLYGON ((0 0, 1 0, 1 1, 0 1, 0 0))")),
-        Row(2, wktReader.read("POINT Z(1 2 3)"), wktReader.read("POLYGON Z((0 0 2, 1 0 2, 1 1 2, 0 1 2, 0 0 2))")),
-        Row(3, wktReader.read("MULTIPOINT (0 0, 1 1, 2 2)"), wktReader.read("MULTILINESTRING ((0 0, 1 1), (2 2, 3 3))"))
-      )
-      val schema = StructType(Seq(
-        StructField("id", IntegerType, nullable = false),
-        StructField("g0", GeometryUDT, nullable = false),
-        StructField("g1", GeometryUDT, nullable = false)
-      ))
+        Row(
+          1,
+          wktReader.read("POINT (1 2)"),
+          wktReader.read("POLYGON ((0 0, 1 0, 1 1, 0 1, 0 0))")),
+        Row(
+          2,
+          wktReader.read("POINT Z(1 2 3)"),
+          wktReader.read("POLYGON Z((0 0 2, 1 0 2, 1 1 2, 0 1 2, 0 0 2))")),
+        Row(
+          3,
+          wktReader.read("MULTIPOINT (0 0, 1 1, 2 2)"),
+          wktReader.read("MULTILINESTRING ((0 0, 1 1), (2 2, 3 3))")))
+      val schema = StructType(
+        Seq(
+          StructField("id", IntegerType, nullable = false),
+          StructField("g0", GeometryUDT, nullable = false),
+          StructField("g1", GeometryUDT, nullable = false)))
       val df = sparkSession.createDataFrame(testData.asJava, schema).repartition(1)
       val geoParquetSavePath = geoparquetoutputlocation + "/multi_geoms.parquet"
       df.write.format("geoparquet").mode("overwrite").save(geoParquetSavePath)
 
       // Find parquet files in geoParquetSavePath directory and validate their metadata
       validateGeoParquetMetadata(geoParquetSavePath) { geo =>
-        implicit val formats : org.json4s.Formats = org.json4s.DefaultFormats
+        implicit val formats: org.json4s.Formats = org.json4s.DefaultFormats
         val version = (geo \ "version").extract[String]
         assert(version == GeoParquetMetaData.VERSION)
         val g0Types = (geo \ "columns" \ "g0" \ "geometry_types").extract[Seq[String]]
@@ -169,10 +224,10 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
     }
 
     it("GeoParquet save should work with empty dataframes") {
-      val schema = StructType(Seq(
-        StructField("id", IntegerType, nullable = false),
-        StructField("g", GeometryUDT, nullable = false)
-      ))
+      val schema = StructType(
+        Seq(
+          StructField("id", IntegerType, nullable = false),
+          StructField("g", GeometryUDT, nullable = false)))
       val df = sparkSession.createDataFrame(Collections.emptyList[Row](), schema)
       val geoParquetSavePath = geoparquetoutputlocation + "/empty.parquet"
       df.write.format("geoparquet").mode("overwrite").save(geoParquetSavePath)
@@ -190,10 +245,10 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
     }
 
     it("GeoParquet save should work with snake_case column names") {
-      val schema = StructType(Seq(
-        StructField("id", IntegerType, nullable = false),
-        StructField("geom_column", GeometryUDT, nullable = false)
-      ))
+      val schema = StructType(
+        Seq(
+          StructField("id", IntegerType, nullable = false),
+          StructField("geom_column", GeometryUDT, nullable = false)))
       val df = sparkSession.createDataFrame(Collections.emptyList[Row](), schema)
       val geoParquetSavePath = geoparquetoutputlocation + "/snake_case_column_name.parquet"
       df.write.format("geoparquet").mode("overwrite").save(geoParquetSavePath)
@@ -205,10 +260,10 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
     }
 
     it("GeoParquet save should work with camelCase column names") {
-      val schema = StructType(Seq(
-        StructField("id", IntegerType, nullable = false),
-        StructField("geomColumn", GeometryUDT, nullable = false)
-      ))
+      val schema = StructType(
+        Seq(
+          StructField("id", IntegerType, nullable = false),
+          StructField("geomColumn", GeometryUDT, nullable = false)))
       val df = sparkSession.createDataFrame(Collections.emptyList[Row](), schema)
       val geoParquetSavePath = geoparquetoutputlocation + "/camel_case_column_name.parquet"
       df.write.format("geoparquet").mode("overwrite").save(geoParquetSavePath)
@@ -270,10 +325,12 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
           |}
           |""".stripMargin
       var geoParquetSavePath = geoparquetoutputlocation + "/gp_custom_meta.parquet"
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.version", "10.9.8")
         .option("geoparquet.crs", projjson)
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       val df2 = sparkSession.read.format("geoparquet").load(geoParquetSavePath)
       assert(df2.count() == df.count())
 
@@ -289,9 +346,11 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
 
       // Setting crs to null explicitly
       geoParquetSavePath = geoparquetoutputlocation + "/gp_crs_null.parquet"
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.crs", "null")
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       val df3 = sparkSession.read.format("geoparquet").load(geoParquetSavePath)
       assert(df3.count() == df.count())
 
@@ -304,9 +363,11 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
 
       // Setting crs to "" to omit crs
       geoParquetSavePath = geoparquetoutputlocation + "/gp_crs_omit.parquet"
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.crs", "")
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       validateGeoParquetMetadata(geoParquetSavePath) { geo =>
         implicit val formats: org.json4s.Formats = org.json4s.DefaultFormats
         val columnName = (geo \ "primary_column").extract[String]
@@ -318,13 +379,15 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
     it("GeoParquet save should support specifying per-column CRS") {
       val wktReader = new WKTReader()
       val testData = Seq(
-        Row(1, wktReader.read("POINT (1 2)"), wktReader.read("POLYGON ((0 0, 1 0, 1 1, 0 1, 0 0))"))
-      )
-      val schema = StructType(Seq(
-        StructField("id", IntegerType, nullable = false),
-        StructField("g0", GeometryUDT, nullable = false),
-        StructField("g1", GeometryUDT, nullable = false)
-      ))
+        Row(
+          1,
+          wktReader.read("POINT (1 2)"),
+          wktReader.read("POLYGON ((0 0, 1 0, 1 1, 0 1, 0 0))")))
+      val schema = StructType(
+        Seq(
+          StructField("id", IntegerType, nullable = false),
+          StructField("g0", GeometryUDT, nullable = false),
+          StructField("g1", GeometryUDT, nullable = false)))
       val df = sparkSession.createDataFrame(testData.asJava, schema).repartition(1)
 
       val projjson0 =
@@ -426,10 +489,12 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
           |""".stripMargin
 
       val geoParquetSavePath = geoparquetoutputlocation + "/multi_geoms_with_custom_crs.parquet"
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.crs", projjson0)
         .option("geoparquet.crs.g1", projjson1)
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       validateGeoParquetMetadata(geoParquetSavePath) { geo =>
         val g0Crs = geo \ "columns" \ "g0" \ "crs"
         val g1Crs = geo \ "columns" \ "g1" \ "crs"
@@ -438,9 +503,11 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
       }
 
       // Write without fallback CRS for g0
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.crs.g1", projjson1)
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       validateGeoParquetMetadata(geoParquetSavePath) { geo =>
         val g0Crs = geo \ "columns" \ "g0" \ "crs"
         val g1Crs = geo \ "columns" \ "g1" \ "crs"
@@ -449,10 +516,12 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
       }
 
       // Fallback CRS is omitting CRS
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.crs", "")
         .option("geoparquet.crs.g1", projjson1)
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       validateGeoParquetMetadata(geoParquetSavePath) { geo =>
         val g0Crs = geo \ "columns" \ "g0" \ "crs"
         val g1Crs = geo \ "columns" \ "g1" \ "crs"
@@ -461,10 +530,12 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
       }
 
       // Write with CRS, explicitly set CRS to null for g1
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.crs", projjson0)
         .option("geoparquet.crs.g1", "null")
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       validateGeoParquetMetadata(geoParquetSavePath) { geo =>
         val g0Crs = geo \ "columns" \ "g0" \ "crs"
         val g1Crs = geo \ "columns" \ "g1" \ "crs"
@@ -473,10 +544,12 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
       }
 
       // Write with CRS, explicitly omit CRS for g1
-      df.write.format("geoparquet")
+      df.write
+        .format("geoparquet")
         .option("geoparquet.crs", projjson0)
         .option("geoparquet.crs.g1", "")
-        .mode("overwrite").save(geoParquetSavePath)
+        .mode("overwrite")
+        .save(geoParquetSavePath)
       validateGeoParquetMetadata(geoParquetSavePath) { geo =>
         val g0Crs = geo \ "columns" \ "g0" \ "crs"
         val g1Crs = geo \ "columns" \ "g1" \ "crs"
@@ -494,7 +567,8 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
 
     it("GeoParquet load with spatial predicates") {
       val df = sparkSession.read.format("geoparquet").load(geoparquetdatalocation1)
-      val rows = df.where(ST_Intersects(ST_Point(35.174722, -6.552465), col("geometry"))).collect()
+      val rows =
+        df.where(ST_Intersects(ST_Point(35.174722, -6.552465), col("geometry"))).collect()
       assert(rows.length == 1)
       assert(rows(0).getAs[String]("name") == "Tanzania")
     }
@@ -506,12 +580,18 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
       // 1000 records.
       val dfIds = (0 until 10000).toDF("id")
       val dfGeom = dfIds
-        .withColumn("bbox", expr("struct(id as minx, id as miny, id + 1 as maxx, id + 1 as maxy)"))
+        .withColumn(
+          "bbox",
+          expr("struct(id as minx, id as miny, id + 1 as maxx, id + 1 as maxy)"))
         .withColumn("geom", expr("ST_PolygonFromEnvelope(id, id, id + 1, id + 1)"))
         .withColumn("part_id", expr("CAST(id / 1000 AS INTEGER)"))
         .coalesce(1)
       val geoParquetSavePath = geoparquetoutputlocation + "/gp_with_bbox.parquet"
-      dfGeom.write.partitionBy("part_id").format("geoparquet").mode("overwrite").save(geoParquetSavePath)
+      dfGeom.write
+        .partitionBy("part_id")
+        .format("geoparquet")
+        .mode("overwrite")
+        .save(geoParquetSavePath)
 
       val sparkListener = new SparkListener() {
         val recordsRead = new AtomicLong(0)
@@ -535,7 +615,8 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
 
         // Reading these files using spatial filter. This should only read two of the files.
         sparkListener.reset()
-        df.where(ST_Intersects(ST_PolygonFromEnvelope(7010, 7010, 8100, 8100), col("geom"))).count()
+        df.where(ST_Intersects(ST_PolygonFromEnvelope(7010, 7010, 8100, 8100), col("geom")))
+          .count()
         assert(sparkListener.recordsRead.get() <= 2000)
       } finally {
         sparkSession.sparkContext.removeSparkListener(sparkListener)
@@ -543,7 +624,10 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
     }
 
     it("Ready legacy parquet files written by Apache Sedona <= 1.3.1-incubating") {
-      val df = sparkSession.read.format("geoparquet").option("legacyMode", "true").load(legacyparquetdatalocation)
+      val df = sparkSession.read
+        .format("geoparquet")
+        .option("legacyMode", "true")
+        .load(legacyparquetdatalocation)
       val rows = df.collect()
       assert(rows.nonEmpty)
       rows.foreach { row =>
@@ -554,14 +638,108 @@ class geoparquetIOTests extends TestBaseScala with BeforeAndAfterAll {
         assert(structGeom.getAs[AnyRef]("g1").isInstanceOf[Geometry])
       }
     }
+
+    it("GeoParquet supports writing covering metadata") {
+      val df = sparkSession
+        .range(0, 100)
+        .toDF("id")
+        .withColumn("id", expr("CAST(id AS DOUBLE)"))
+        .withColumn("geometry", expr("ST_Point(id, id + 1)"))
+        .withColumn(
+          "test_cov",
+          expr("struct(id AS xmin, id + 1 AS ymin, id AS xmax, id + 1 AS ymax)"))
+      val geoParquetSavePath = geoparquetoutputlocation + "/gp_with_covering_metadata.parquet"
+      df.write
+        .format("geoparquet")
+        .option("geoparquet.covering", "test_cov")
+        .mode("overwrite")
+        .save(geoParquetSavePath)
+      validateGeoParquetMetadata(geoParquetSavePath) { geo =>
+        implicit val formats: org.json4s.Formats = org.json4s.DefaultFormats
+        val coveringJsValue = geo \ "columns" \ "geometry" \ "covering"
+        val covering = coveringJsValue.extract[Covering]
+        assert(covering.bbox.xmin == Seq("test_cov", "xmin"))
+        assert(covering.bbox.ymin == Seq("test_cov", "ymin"))
+        assert(covering.bbox.xmax == Seq("test_cov", "xmax"))
+        assert(covering.bbox.ymax == Seq("test_cov", "ymax"))
+      }
+
+      df.write
+        .format("geoparquet")
+        .option("geoparquet.covering.geometry", "test_cov")
+        .mode("overwrite")
+        .save(geoParquetSavePath)
+      validateGeoParquetMetadata(geoParquetSavePath) { geo =>
+        implicit val formats: org.json4s.Formats = org.json4s.DefaultFormats
+        val coveringJsValue = geo \ "columns" \ "geometry" \ "covering"
+        val covering = coveringJsValue.extract[Covering]
+        assert(covering.bbox.xmin == Seq("test_cov", "xmin"))
+        assert(covering.bbox.ymin == Seq("test_cov", "ymin"))
+        assert(covering.bbox.xmax == Seq("test_cov", "xmax"))
+        assert(covering.bbox.ymax == Seq("test_cov", "ymax"))
+      }
+    }
+
+    it("GeoParquet supports writing covering metadata for multiple columns") {
+      val df = sparkSession
+        .range(0, 100)
+        .toDF("id")
+        .withColumn("id", expr("CAST(id AS DOUBLE)"))
+        .withColumn("geom1", expr("ST_Point(id, id + 1)"))
+        .withColumn(
+          "test_cov1",
+          expr("struct(id AS xmin, id + 1 AS ymin, id AS xmax, id + 1 AS ymax)"))
+        .withColumn("geom2", expr("ST_Point(10 * id, 10 * id + 1)"))
+        .withColumn(
+          "test_cov2",
+          expr(
+            "struct(10 * id AS xmin, 10 * id + 1 AS ymin, 10 * id AS xmax, 10 * id + 1 AS ymax)"))
+      val geoParquetSavePath = geoparquetoutputlocation + "/gp_with_covering_metadata.parquet"
+      df.write
+        .format("geoparquet")
+        .option("geoparquet.covering.geom1", "test_cov1")
+        .option("geoparquet.covering.geom2", "test_cov2")
+        .mode("overwrite")
+        .save(geoParquetSavePath)
+      validateGeoParquetMetadata(geoParquetSavePath) { geo =>
+        implicit val formats: org.json4s.Formats = org.json4s.DefaultFormats
+        Seq(("geom1", "test_cov1"), ("geom2", "test_cov2")).foreach {
+          case (geomName, coveringName) =>
+            val coveringJsValue = geo \ "columns" \ geomName \ "covering"
+            val covering = coveringJsValue.extract[Covering]
+            assert(covering.bbox.xmin == Seq(coveringName, "xmin"))
+            assert(covering.bbox.ymin == Seq(coveringName, "ymin"))
+            assert(covering.bbox.xmax == Seq(coveringName, "xmax"))
+            assert(covering.bbox.ymax == Seq(coveringName, "ymax"))
+        }
+      }
+
+      df.write
+        .format("geoparquet")
+        .option("geoparquet.covering.geom2", "test_cov2")
+        .mode("overwrite")
+        .save(geoParquetSavePath)
+      validateGeoParquetMetadata(geoParquetSavePath) { geo =>
+        implicit val formats: org.json4s.Formats = org.json4s.DefaultFormats
+        assert(geo \ "columns" \ "geom1" \ "covering" == org.json4s.JNothing)
+        val coveringJsValue = geo \ "columns" \ "geom2" \ "covering"
+        val covering = coveringJsValue.extract[Covering]
+        assert(covering.bbox.xmin == Seq("test_cov2", "xmin"))
+        assert(covering.bbox.ymin == Seq("test_cov2", "ymin"))
+        assert(covering.bbox.xmax == Seq("test_cov2", "xmax"))
+        assert(covering.bbox.ymax == Seq("test_cov2", "ymax"))
+      }
+    }
   }
 
   def validateGeoParquetMetadata(path: String)(body: org.json4s.JValue => Unit): Unit = {
     val parquetFiles = new File(path).listFiles().filter(_.getName.endsWith(".parquet"))
     parquetFiles.foreach { filePath =>
-      val metadata = ParquetFileReader.open(
-        HadoopInputFile.fromPath(new Path(filePath.getPath), new Configuration()))
-        .getFooter.getFileMetaData.getKeyValueMetaData
+      val metadata = ParquetFileReader
+        .open(HadoopInputFile.fromPath(new Path(filePath.getPath), new Configuration()))
+        .getFooter
+        .getFileMetaData
+        .getKeyValueMetaData
       assert(metadata.containsKey("geo"))
       val geo = parseJson(metadata.get("geo"))
       body(geo)
