@@ -1,15 +1,20 @@
 /*
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.apache.spark.sql.execution.datasources.parquet
 
@@ -46,7 +51,12 @@ import scala.util.Failure
 import scala.util.Try
 
 class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
-  extends ParquetFileFormat with GeoParquetFileFormatBase with FileFormat with DataSourceRegister with Logging with Serializable {
+    extends ParquetFileFormat
+    with GeoParquetFileFormatBase
+    with FileFormat
+    with DataSourceRegister
+    with Logging
+    with Serializable {
 
   def this() = this(None)
 
@@ -55,21 +65,29 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
 
   override def hashCode(): Int = getClass.hashCode()
 
+  override def toString(): String = {
+    // HACK: This is the only place we can inject spatial filter information into the described query plan.
+    // Please see org.apache.spark.sql.execution.DataSourceScanExec#simpleString for more details.
+    "GeoParquet" + spatialFilter
+      .map(filter => " with spatial filter [" + filter.simpleString + "]")
+      .getOrElse("")
+  }
+
   def withSpatialPredicates(spatialFilter: GeoParquetSpatialFilter): GeoParquetFileFormat =
     new GeoParquetFileFormat(Some(spatialFilter))
 
   override def inferSchema(
-                            sparkSession: SparkSession,
-                            parameters: Map[String, String],
-                            files: Seq[FileStatus]): Option[StructType] = {
+      sparkSession: SparkSession,
+      parameters: Map[String, String],
+      files: Seq[FileStatus]): Option[StructType] = {
     GeoParquetUtils.inferSchema(sparkSession, parameters, files)
   }
 
   override def prepareWrite(
-    sparkSession: SparkSession,
-    job: Job,
-    options: Map[String, String],
-    dataSchema: StructType): OutputWriterFactory = {
+      sparkSession: SparkSession,
+      job: Job,
+      options: Map[String, String],
+      dataSchema: StructType): OutputWriterFactory = {
     val parquetOptions = new ParquetOptions(options, sparkSession.sessionState.conf)
 
     val conf = ContextUtil.getConfiguration(job)
@@ -81,16 +99,15 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
         classOf[OutputCommitter])
 
     if (conf.get(SQLConf.PARQUET_OUTPUT_COMMITTER_CLASS.key) == null) {
-      logInfo("Using default output committer for Parquet: " +
-        classOf[ParquetOutputCommitter].getCanonicalName)
+      logInfo(
+        "Using default output committer for Parquet: " +
+          classOf[ParquetOutputCommitter].getCanonicalName)
     } else {
-      logInfo("Using user defined output committer for Parquet: " + committerClass.getCanonicalName)
+      logInfo(
+        "Using user defined output committer for Parquet: " + committerClass.getCanonicalName)
     }
 
-    conf.setClass(
-      SQLConf.OUTPUT_COMMITTER_CLASS.key,
-      committerClass,
-      classOf[OutputCommitter])
+    conf.setClass(SQLConf.OUTPUT_COMMITTER_CLASS.key, committerClass, classOf[OutputCommitter])
 
     // We're not really using `ParquetOutputFormat[Row]` for writing data here, because we override
     // it in `ParquetOutputWriter` to support appending and dynamic partitioning.  The reason why
@@ -114,7 +131,8 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
       sparkSession.sessionState.conf.parquetOutputTimestampType.toString)
 
     try {
-      val fieldIdWriteEnabled = SQLConf.get.getConfString("spark.sql.parquet.fieldId.write.enabled")
+      val fieldIdWriteEnabled =
+        SQLConf.get.getConfString("spark.sql.parquet.fieldId.write.enabled")
       conf.set("spark.sql.parquet.fieldId.write.enabled", fieldIdWriteEnabled)
     } catch {
       case e: NoSuchElementException => ()
@@ -132,18 +150,19 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
     if (ParquetOutputFormat.getJobSummaryLevel(conf) != JobSummaryLevel.NONE
       && !classOf[ParquetOutputCommitter].isAssignableFrom(committerClass)) {
       // output summary is requested, but the class is not a Parquet Committer
-      logWarning(s"Committer $committerClass is not a ParquetOutputCommitter and cannot" +
-        s" create job summaries. " +
-        s"Set Parquet option ${ParquetOutputFormat.JOB_SUMMARY_LEVEL} to NONE.")
+      logWarning(
+        s"Committer $committerClass is not a ParquetOutputCommitter and cannot" +
+          s" create job summaries. " +
+          s"Set Parquet option ${ParquetOutputFormat.JOB_SUMMARY_LEVEL} to NONE.")
     }
 
     conf.set(ParquetOutputFormat.WRITE_SUPPORT_CLASS, classOf[GeoParquetWriteSupport].getName)
 
     new OutputWriterFactory {
       override def newInstance(
-        path: String,
-        dataSchema: StructType,
-        context: TaskAttemptContext): OutputWriter = {
+          path: String,
+          dataSchema: StructType,
+          context: TaskAttemptContext): OutputWriter = {
         new ParquetOutputWriter(path, context)
       }
 
@@ -154,20 +173,16 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
   }
 
   override def buildReaderWithPartitionValues(
-                                               sparkSession: SparkSession,
-                                               dataSchema: StructType,
-                                               partitionSchema: StructType,
-                                               requiredSchema: StructType,
-                                               filters: Seq[Filter],
-                                               options: Map[String, String],
-                                               hadoopConf: Configuration): (PartitionedFile) => Iterator[InternalRow] = {
+      sparkSession: SparkSession,
+      dataSchema: StructType,
+      partitionSchema: StructType,
+      requiredSchema: StructType,
+      filters: Seq[Filter],
+      options: Map[String, String],
+      hadoopConf: Configuration): (PartitionedFile) => Iterator[InternalRow] = {
     hadoopConf.set(ParquetInputFormat.READ_SUPPORT_CLASS, classOf[ParquetReadSupport].getName)
-    hadoopConf.set(
-      ParquetReadSupport.SPARK_ROW_REQUESTED_SCHEMA,
-      requiredSchema.json)
-    hadoopConf.set(
-      ParquetWriteSupport.SPARK_ROW_SCHEMA,
-      requiredSchema.json)
+    hadoopConf.set(ParquetReadSupport.SPARK_ROW_REQUESTED_SCHEMA, requiredSchema.json)
+    hadoopConf.set(ParquetWriteSupport.SPARK_ROW_SCHEMA, requiredSchema.json)
     hadoopConf.set(
       SQLConf.SESSION_LOCAL_TIMEZONE.key,
       sparkSession.sessionState.conf.sessionLocalTimeZone)
@@ -209,7 +224,8 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
     val pushDownDate = sqlConf.parquetFilterPushDownDate
     val pushDownTimestamp = sqlConf.parquetFilterPushDownTimestamp
     val pushDownDecimal = sqlConf.parquetFilterPushDownDecimal
-    val parquetFilterPushDownStringPredicate = sqlConf.parquetFilterPushDownStringPredicate // .parquetFilterPushDownStringStartWith
+    val parquetFilterPushDownStringPredicate =
+      sqlConf.parquetFilterPushDownStringPredicate // .parquetFilterPushDownStringStartWith
     val pushDownInFilterThreshold = sqlConf.parquetFilterPushDownInFilterThreshold
     val isCaseSensitive = sqlConf.caseSensitiveAnalysis
 
@@ -239,8 +255,15 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
       // Try to push down filters when filter push-down is enabled.
       val pushed = if (enableParquetFilterPushDown) {
         val parquetSchema = footerFileMetaData.getSchema
-        val parquetFilters = new ParquetFilters(parquetSchema, pushDownDate, pushDownTimestamp,
-          pushDownDecimal, parquetFilterPushDownStringPredicate, pushDownInFilterThreshold, isCaseSensitive, datetimeRebaseSpec)
+        val parquetFilters = new ParquetFilters(
+          parquetSchema,
+          pushDownDate,
+          pushDownTimestamp,
+          pushDownDecimal,
+          parquetFilterPushDownStringPredicate,
+          pushDownInFilterThreshold,
+          isCaseSensitive,
+          datetimeRebaseSpec)
         filters
           // Collects all converted Parquet filter predicates. Notice that not all predicates can be
           // converted (`ParquetFilters.createFilter` returns an `Option`). That's why a `flatMap`
@@ -252,9 +275,10 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
       }
 
       // Prune file scans using pushed down spatial filters and per-column bboxes in geoparquet metadata
-      val shouldScanFile = GeoParquetMetaData.parseKeyValueMetaData(footerFileMetaData.getKeyValueMetaData).forall {
-        metadata => spatialFilter.forall(_.evaluate(metadata.columns))
-      }
+      val shouldScanFile =
+        GeoParquetMetaData.parseKeyValueMetaData(footerFileMetaData.getKeyValueMetaData).forall {
+          metadata => spatialFilter.forall(_.evaluate(metadata.columns))
+        }
       if (!shouldScanFile) {
         // The entire file is pruned so that we don't need to scan this file.
         Seq.empty[InternalRow].iterator
@@ -291,7 +315,8 @@ class GeoParquetFileFormat(val spatialFilter: Option[GeoParquetSpatialFilter])
         }
         val taskContext = Option(TaskContext.get())
         if (enableVectorizedReader) {
-          logWarning(s"GeoParquet currently does not support vectorized reader. Falling back to parquet-mr")
+          logWarning(
+            s"GeoParquet currently does not support vectorized reader. Falling back to parquet-mr")
         }
         logDebug(s"Falling back to parquet-mr")
         // ParquetRecordReader returns InternalRow
@@ -338,19 +363,19 @@ object GeoParquetFileFormat extends Logging {
    *
    * Note that locality is not taken into consideration here because:
    *
-   *  1. For a single Parquet part-file, in most cases the footer only resides in the last block of
-   *     that file.  Thus we only need to retrieve the location of the last block.  However, Hadoop
-   *     `FileSystem` only provides API to retrieve locations of all blocks, which can be
-   *     potentially expensive.
+   *   1. For a single Parquet part-file, in most cases the footer only resides in the last block
+   *      of that file. Thus we only need to retrieve the location of the last block. However,
+   *      Hadoop `FileSystem` only provides API to retrieve locations of all blocks, which can be
+   *      potentially expensive.
    *
-   *  2. This optimization is mainly useful for S3, where file metadata operations can be pretty
-   *     slow.  And basically locality is not available when using S3 (you can't run computation on
-   *     S3 nodes).
+   * 2. This optimization is mainly useful for S3, where file metadata operations can be pretty
+   * slow. And basically locality is not available when using S3 (you can't run computation on S3
+   * nodes).
    */
   def mergeSchemasInParallel(
-                              parameters: Map[String, String],
-                              filesToTouch: Seq[FileStatus],
-                              sparkSession: SparkSession): Option[StructType] = {
+      parameters: Map[String, String],
+      filesToTouch: Seq[FileStatus],
+      sparkSession: SparkSession): Option[StructType] = {
     val assumeBinaryIsString = sparkSession.sessionState.conf.isParquetBinaryAsString
     val assumeInt96IsTimestamp = sparkSession.sessionState.conf.isParquetINT96AsTimestamp
 
@@ -371,43 +396,50 @@ object GeoParquetFileFormat extends Logging {
     SchemaMergeUtils.mergeSchemasInParallel(sparkSession, parameters, filesToTouch, reader)
   }
 
-  private def readSchemaFromFooter(footer: Footer, keyValueMetaData: java.util.Map[String, String],
-                                   converter: GeoParquetToSparkSchemaConverter,
-                                   parameters: Map[String, String]): StructType = {
+  private def readSchemaFromFooter(
+      footer: Footer,
+      keyValueMetaData: java.util.Map[String, String],
+      converter: GeoParquetToSparkSchemaConverter,
+      parameters: Map[String, String]): StructType = {
     val fileMetaData = footer.getParquetMetadata.getFileMetaData
-    fileMetaData
-      .getKeyValueMetaData
-      .asScala.toMap
+    fileMetaData.getKeyValueMetaData.asScala.toMap
       .get(ParquetReadSupport.SPARK_METADATA_KEY)
       .flatMap(schema => deserializeSchemaString(schema, keyValueMetaData, parameters))
       .getOrElse(converter.convert(fileMetaData.getSchema))
   }
 
-  private def deserializeSchemaString(schemaString: String, keyValueMetaData: java.util.Map[String, String],
-                                      parameters: Map[String, String]): Option[StructType] = {
+  private def deserializeSchemaString(
+      schemaString: String,
+      keyValueMetaData: java.util.Map[String, String],
+      parameters: Map[String, String]): Option[StructType] = {
     // Tries to deserialize the schema string as JSON first, then falls back to the case class
     // string parser (data generated by older versions of Spark SQL uses this format).
-    val schemaOpt = Try(DataType.fromJson(schemaString).asInstanceOf[StructType]).recover {
-      case _: Throwable =>
+    val schemaOpt = Try(DataType.fromJson(schemaString).asInstanceOf[StructType])
+      .recover { case _: Throwable =>
         logInfo(
           "Serialized Spark schema in Parquet key-value metadata is not in JSON format, " +
             "falling back to the deprecated DataType.fromCaseClassString parser.")
         LegacyTypeStringParser.parseString(schemaString).asInstanceOf[StructType]
-    }.recoverWith {
-      case cause: Throwable =>
+      }
+      .recoverWith { case cause: Throwable =>
         logWarning(
           "Failed to parse and ignored serialized Spark schema in " +
-            s"Parquet key-value metadata:\n\t$schemaString", cause)
+            s"Parquet key-value metadata:\n\t$schemaString",
+          cause)
         Failure(cause)
-    }.toOption
+      }
+      .toOption
 
-    schemaOpt.map(schema => replaceGeometryColumnWithGeometryUDT(schema, keyValueMetaData, parameters))
+    schemaOpt.map(schema =>
+      replaceGeometryColumnWithGeometryUDT(schema, keyValueMetaData, parameters))
   }
 
-  private def replaceGeometryColumnWithGeometryUDT(schema: StructType,
-                                                   keyValueMetaData: java.util.Map[String, String],
-                                                   parameters: Map[String, String]): StructType = {
-    val geoParquetMetaData: GeoParquetMetaData = GeoParquetUtils.parseGeoParquetMetaData(keyValueMetaData, parameters)
+  private def replaceGeometryColumnWithGeometryUDT(
+      schema: StructType,
+      keyValueMetaData: java.util.Map[String, String],
+      parameters: Map[String, String]): StructType = {
+    val geoParquetMetaData: GeoParquetMetaData =
+      GeoParquetUtils.parseGeoParquetMetaData(keyValueMetaData, parameters)
     val fields = schema.fields.map { field =>
       field.dataType match {
         case _: BinaryType if geoParquetMetaData.columns.contains(field.name) =>
